#!/bin/sh

# dbtres:
# Get header parameters from compressed FITS files
# By Doug Mink, December 5, 2008

if test -z $1
    then
    echo dbtres: Get header parameters from compressed TRES files
    echo "       usage: dbtres [-v][-d directory] fits.file"
    echo "		dbtres [-v][-d directory] @fits.file.list.file"
    echo "		dbtres [-v][-d directory] -date yyyy.mmdd"
    echo "		-v: log processing steps"
    exit
    fi

vb=
hd=
arcdir="/data/tres/Archive/rawdata/data"
dir=
datedir=

# Loop through arguments
if test -r templist
    then
    /bin/rm templist
    fi

args="-p -b object imagetyp ra dec mjd apfib fibkey exptime comptime program"
args1=`echo $args`

while test $1
    do

    # Check for verbose flag
    if test $1 = "-v"
	then
	#echo verbose mode
	vb=true

    # Check for column header flag
    elif test $1 = "-h"
	then
	args1=`echo "-h" $args`

    # Save directory path if present
    elif test $1 = "-date"
	then
	shift
	datedir=$1
	fromfile=`echo $datedir".dir"`
	if test -z $dir
	    then
	    year=`echo $datedir | cut -b 1-4`
	    if test $year = "2007"
		then
		dir=`echo $arcdir"/2007"`
	    else
		dir=$arcdir
	    fi
	fi
	datadir=`echo $dir"/"$datedir`
	ls $datadir | grep fits | sed -e s/^/$datedir\\\// >$fromfile
	if test $vb
	    then
	    echo Data files from $datadir
	    echo "      listed to" $fromfile
	    fi

    # Save directory path if present
    elif test $1 = "-d"
	then
	shift
	dir=$1
	if test $vb
	    then
	    echo Reading from directory $dir
	    fi

    # Set full pathname to list of files if list file encountered
    else
	listfile=`isimlist $1`
	isfits=`echo $1 | grep fits`
	if test $listfile
	    then
	    fromfile=$listfile
	    if test $vb
		then
		echo Reading from list of FITS files in $fromfile
		fi

    # If FITS file, add to list of files to check
	elif test $isfits
	    then
	    if test $datedir
		then
		echo $datedir"/"$1 > templist
	    else
		echo $1 > templist
	    fi
	    fromfile=`echo templist`

    # Add argument to list
	else
	    args=`echo $args $1`
	fi
    fi
    shift
    done

if test $vb
    then
    echo gethead "<file>" $args
    echo "where <file> is"
    cat $fromfile
    fi
    apfib=
    fibkey=

for file in `cat $fromfile`
    do
    if test $vb
	then
	echo File is \"$file\"
	fi
    bzcomp=`echo $file | grep \.bz2`
    gzcomp=`echo $file | grep \.gz`
    zcomp=`echo $file | grep \.Z`

# Prepend directory to file if needed
    if test $dir
	then
	filein=`echo $dir/$file`
    else
	filein=$file
    fi

# Extract header into file yyyy.mmdd.<file>.head
    if test $bzcomp
	then
	filehead=`echo $file | sed -e s/\\\//./ | sed -e s/fits\.bz2/head/`
	if test $vb
	    then
	    echo bzcat $filein to read header
	    fi
	bzcat $filein | imhead -f stdin >$filehead
    elif test $gzcomp
	then
	filehead=`echo $file | sed -e s/\\\//./ | sed -e s/fits\.gz/head/`
	if test $vb
	    then
	    echo zcat $filein to read header
	    fi
	zcat $filein | imhead -f stdin >$filehead
    elif test $zcomp
	then
	filehead=`echo $file | sed -e s/\\\//./ | sed -e s/fits\.Z/head/`
	if test $vb
	    then
	    echo zcat $filein to read header
	    fi
	zcat $filein | imhead -f stdin > $filehead
    else
	filehead=`echo $file | sed -e s/\\\//./ | sed -e s/fits/head/`
	imhead -f $filein >$filehead
    fi

    apfib=`gethead $filehead apfib`
    if test $apfib
	then
	if test $vb
	    then
	    echo APFIB is $apfib
	    fi
    else
	if test $vb
	    then
	    echo Creating APFIB
	    fi
	fibkey=`gethead $filehead fibkey`
	if test -z $fibkey
	    then
	    if test $vb
		then
		echo computing FIBKEY
		fi
	    object=`gethead $filehead object`
	    imagetyp=`gethead $filehead imagetyp`
	    if test $vb
		then
		echo "IMAGETYP=" $imagetyp", OBJECT=" $object
		fi
	    if test -z $imagetyp
		then
		imagetyp="OBJECT"
		sethead $filehead imagetyp=$imagetyp
		fi
	    peri=`gethead $filehead peri`
	    if test $imagetyp = "FLAT"
		then
		if test $peri = "IN"
		    then
		    fibkey="FFN"
		    fiber="12"
		else
		    fibkey="NNF"
		    fiber="3"
		fi
	    elif test $imagetyp = "COMP"
		then
		if test $peri = "IN"
		    then
		    fibkey="CCN"
		    fiber="12"
		else
		    fibkey="NNC"
		    fiber="3"
		fi
	    elif test $imagetyp = "OBJECT"
		then
		if test $object = "COMP"
		    then
		    if test $peri = "IN"
			then
			fibkey="CCN"
			fiber="12"
		    else
			fibkey="NNC"
			fiber="3"
		    fi
		else
		    osswitch=`gethead $filehead ocswitch`
		    if test $osswitch
			then
			if test $osswitch = "YES"
			    then
			    fibkey="SO"
			    fiber="12"
			else
			    fibkey="OS"
			    fiber="12"
			fi
		    else
			osswitch=`gethead $filehead osswitch`
			if test $osswitch = "YES"
			    then
			    fibkey="SO"
			    fiber="12"
			else
			    fibkey="OS"
			    fiber="12"
			fi
		    fi
		    if test peri = "IN"
			then
			comptime=`gethead $filehead comptime`
			if test comptime = "0.0"
			    then
			    fibkey=`echo $fibkey"N"`
			else
			    fibkey=`echo $fibkey"C"`
			    fiber="123"
			fi
		    else
			fibkey=`echo $fibkey"N"`
		    fi
		fi
	    else
		fibkey="NNN"
	    fi
	    sethead $filehead fibkey=$fibkey
	    if test $vb
		then
		echo FIBKEY from keywords is $fibkey
		fi
	else
	    if test $vb
		then
		echo FIBKEY from header is $fibkey
		fi
	    if test $fibkey = "CCN"
		then
		fiber="12"
	    elif test $fibkey = "NNC"
		then
		fiber="3"
	    elif test $fibkey = "OSN"
		then
		fiber="12"
	    elif test $fibkey = "SON"
		then
		fiber="12"
	    elif test $fibkey = "SOC"
		then
		fiber="123"
	    elif test $fibkey = "OSC"
		then
		fiber="123"
	    elif test $fibkey = "FFN"
		then
		fiber="12"
	    elif test $fibkey = "NNF"
		then
		fiber="3"
	    else
		fiber=
	    fi
	fi 

# Assemble APFIB keyword from aperture size, binning, and fiber
	if test $vb
	    then
	    echo FIBKEY is $fibkey, fiber is $fiber, computing APFIB
	    fi
	fibsize=`gethead $filehead fibsize`
	if test $fibsize = "1.4"
	    then
	    apsize="s"
	elif test $fibsize = "2.3"
	    then
	    apsize="m"
	else
	    apsize="l"
	fi
	binning=`gethead $filehead ccdsum,1`
	if test $binning
	    then
	    if test $binning = "2"
		then
		apfib=`echo $apsize"b"$fiber`
	    else
		apfib=`echo $apsize$fiber`
	    fi
	else
	    apfib=`echo $apsize$fiber`
	fi
	sethead $filehead apfib=$apfib
	if test $vb
	    then
	    echo FIBKEY is $fibkey, fiber is $fiber, APFIB is $apfib
	    fi
    fi
    gethead $filehead $args1 | sed -e s/head/fits/
    args1=$args

    if ! $vb
	then
	if test -e $filehead
	    then
	    /bin/rm $filehead
	    fi
	fi

    done

   if test -e $fromfile
	then
	/bin/rm $fromfile
	fi
