# File tres/Scripts/ftres.cl
# December 11, 2008
# By Doug Mink, SAO Telescope Data Center

# FTRES - Process multiple similar TRES flat images from raw data files

procedure ftres (images)

string	images=""	{prompt="Images to process"}
string	datedir=""	{prompt="Date for images"}
string	instdir=""	{prompt="Directory containing date directory"}
bool	fixpreamp=no	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=0.95	{prompt="Gain correction for amp2 to amp1"}
bool	flatten=no	{prompt="Run TFLAT to Flatten image (yes or no)"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir="tresdata$"	{prompt="Directory for normalization file"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
bool	sumspec=no	{prompt="Compute a mean image (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	delimage=no	{prompt="Delete processed image file (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (y or n)"}
bool	saveold=no	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath
file	imout, ims, cdir, tempin, tempout
string	ddir, idir, croot
bool	vb, dt, sr, sv, disp, rc, sum, sumonly, delim, avg
real	ct

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	ddir = datedir
	idir = instdir
	ims = images
	delim = delimage
	ct = threshold
	avg = sumspec

	if (substr (ims,1,1) == "@")
	    fileroot (substr (ims,2,100))
	else
	    fileroot (ims)
	trssum.outroot = fileroot.rootname

	if (ims == "") {
	    print ("FTRES: Reduce multiple similar TRES flat field images, removing cosmic rays")
	    print ("Usage: ftres images with both amps to be corrected")
	    return
	    }

# If input image is a list prepend @ if not already there
	smatch (ims, ".list")
	if (smatch.found) {
	    if (substr (ims,1,1) != '@')
		ims = "@" // images
	    }

	tcheck
	if (tcheck.rawdir)
	    trsdate (tcheck.datedir)

#  Make list file from input file parameter
	trsfiles.instdir = idir
	trsfiles.datedir = ddir
	trsfiles.copyfile = yes
	trsfiles.overwrite = no
	trsfiles (ims)
	tempin = trsfiles.tempin
	tempout = trsfiles.tempout
	ims = "@" // tempout

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.flatten = no
	tproc.cosmic = yes
	tproc.threshold = ct
	tproc.sumspec = "mean"
	tproc.deltemp = dt
	tproc.saveraw = sr
	tproc.saveold = sv
	tproc.verbose = vb
	tcosmic.savebp = sv
	tproc (ims, flatten=no, extract=no, disperse=no)

# Compute average file if requested
	if (avg) {
	    trsavg (ims)
	    if (trsavg.nfiles == 1)
		print ("FTRES: No average computed from one file ",trsavg.outfile)
	    }

	if (delim) {
	    imdelete (ims)
	    }

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

	if (tcheck.rawdir)
	    tdata (datedir=tcheck.datedir)
end

# Jul 16 2008	New task based on CTRES
# Oct 14 2008	Do not run if in raw data directory
# Oct 14 2008	Make default to save raw files
# Oct 15 2008	Add threshold parameter to set cosmic ray threshold
# Oct 16 2008	From raw directory, switch to quick-look directory and back
# Oct 22 2008	Fix bug when reading a single file
# Dec  5 2008	Make @ optional if images is *.list file
# Dec  5 2008	Add sumspec parameter and make average image if yes
# Dec 11 2008	Note that no average is produced from a single input file
