# File tres/Scripts/tgain1.cl
# By Doug Mink after Bill Wyatt
# December 15, 2008

# Run the 2 separate gain corrections for TRES

procedure tgain1 (image)

string image = ""        {prompt="Image to correct"}

real	g1 = 1.0	{prompt="Gain correction for amplifier 1"}
real	g2 = 0.990	{prompt="Gain correction for amplifier 2"}
bool	saveold=yes     {prompt="Save unprocessed image files (y or n)"}
bool	deltemp=yes	{prompt="Save single-extension files (y or n)"}
bool	verbose=yes	{prompt="Verbose output (y or n)"}
bool	debug=no	{prompt="Extra debugging output (y or n)"}

begin

	string imroot, gcroot, imsec, imx
	string gcsect0, gcsect1, gcsect2
	string imsect0, imsect1, imsect2
	file imin, imu
	bool vb, db, dt, sv

	vb = verbose
	db = debug
	dt = deltemp
	sv = saveold
	imin = image
	imx = ".fits"

#  If amplifier gain has already been done, return
	imgetp.default = "0" 
	imsec = imin // "[0]"
	imgetp (imsec, "GAINCOR1")
	if (imgetp.value != "0") {
	    print ("TGAIN1: GAINCORR has already been run on", imin)
	    print ("GAINCORR: ", imgetp.value)
	    imgetp (imsec, "GAINCOR2")
	    print ("GAINCORR: ", imgetp.value)
	    return
	    }
	else {
	    imsec = imin// "[1]"
	    imgetp (imsec, "GAINCORR")
	    if (imgetp.value != "0") {
		print ("TGAIN1: GAINCORR has already been run on", imin)
		print ("GAINCORR: ", imgetp.value)
		return
		}
	    }

	if (vb)
	    printf("TGAIN1: imarith on extensions of %s\n",imin)
	
# Find root name of image
	imroot = imin
	smatch (imroot, imx)
	if (smatch.found) {
	    imroot = substr (imroot,1,smatch.start-1)
	    }
	else {
	    imroot = imin
	    imin = imin // imx
	    }
   
# Set output file name root
	gcroot = imroot // "gc"

# Assign file names for extensions of output files
	gcsect0 = gcroot // "_0.fits"
	if (access (gcsect0))
	    delete (gcsect0)
	gcsect1 = gcroot // "_1.fits"
	if (access (gcsect1))
	    delete (gcsect1)
	gcsect2 = gcroot // "_2.fits"
	if (access (gcsect2))
	    delete (gcsect2)

# Multiply each amplifier image by appropriate gain correction
	imsect0 = imin // "[0]"
	imsect1 = imin // "[1]"
	imsect2 = imin // "[2]"
	imcopy (imsect0, gcsect0, verbose=vb)
	imarith(imsect1, '*', g1, gcsect1, verbose=db)
	imarith(imsect2, '*', g2, gcsect2, verbose=db)

  # Add GAINCORR keyword to header
	hedit.show = no
	hedit (gcsect1, "GAINCORR", (g1), add=yes, verify=no)
	hedit (gcsect2, "GAINCORR", (g2), add=yes, verify=no)

#  Save original file in Ungain/ directory, if requested
	if (sv) {
	    if (!access ("Ungain/"))
		mkdir ("Ungain")
	    imu = "Ungain/" // imin
	    if (access (imu))
		delete (imu)
	    rename (imin, imu)
	    }
	else {
	    delete (imin)
	    }

#  Rejoin result to same name as original file
	mscjoin (gcroot, output=imin, delete=dt, verbose=vb)
	if (vb)
	    print("TGAIN1: rejoined ",imin)

end

# Feb 25 2008	New task to gain correct one image, based on GAINCORRECT
# Dec 15 2008	Check primary header for GAINCOR1 first, then extension header GAINCORR
