# File tres/Scripts/tmodid.cl
# By Doug Mink
# December 15, 2008

# TMODID -- Modify line positions in ecidentify database file

procedure tmodid (dbfin, dbfout)

string dbfin=""		{prompt="File to process"}
string dbfout=""	{prompt="File for output"}

string indir="tresdata$" {prompt="Input directory with database directory"}
string outdir=""	{prompt="Output directory with database directory"}
real shift=0.0		{prompt="Constant pixel shift (0 is none)"}
real scale=1.0		{prompt="Constant pixel scale factor (1 is none)"}
bool keepfit=no		{prompt="Keep dispersion fit (yes or no)"}
bool verbose=yes	{prompt="Log progress (yes or no)"}
bool debug=no		{prompt="Print output (yes or no)"}

begin

    bool vb, db, kf, first
    int i, io, ncoeff, icoeff
    file idbpath, idbdir, odbpath, odbdir
    string idbfile, odbfile, idbroot, odbroot
    string s1, s2, s3, s4, s5, s6, s7, s8
    real pshift, pscale, x, w1, w2, shiftmin

    vb = verbose
    db = debug
    kf = keepfit
    pshift = shift
    if (pshift < 0.000001 && pshift > -0.000001) {
	pshift = 0.0
	}
    pscale = scale

# Input dispersion fit file
    if (indir != "") {
	fixdir (indir)
	idbdir = fixdir.newdir // "database/"
	}
    else {
	idbdir = "database/"
	}
    idbfile = dbfin
    fileroot (idbfile)
    idbroot = fileroot.rootname
    idbpath = idbdir // "ec" // idbroot

# Output dispersion fit file
    if (outdir != "") {
	fixdir (outdir)
	odbdir = fixdir.newdir // "database/"
	}
    else {
	odbdir = "database/"
	}
    odbfile = dbfout
    fileroot (odbfile)
    odbroot = fileroot.rootname
    odbpath = odbdir // "ec" // odbroot

# Make output directory if it does not already exist
    if (!access (odbdir))
	mkdir (obdir)

# Quit if input database entry is not there
    if (!access (idbpath)) {
	print ("TMODID: Database file ",idbpath, " does not exist")
	return
	}

# If output database entry already exists, delete it
    if (access (odbpath))
	delete (odbpath)

# If files have same name and no shift, just copy the original
    if (dbfin == dbfout && pshift == 0.0) {
	copy (idbpath, odbpath)
	if (vb) {
	    print ("TMODID: Copying ",idbpath," to ", odbpath)
	    }
	return
	}

    if (vb) {
	print ("TMODID: Shifting ",idbpath," to ", odbpath, " by ",pshift)
	}
    s1 = ""
    s2 = ""
    s3 = ""
    s4 = ""
    s5 = ""
    s6 = ""
    s7 = ""
    s8 = ""
    ncoeff = 0
    gettime (0.0, timediff=99, usesec+)

# Read through database entry one line at a time
    first = no
    list = idbpath
    while (fscan (list, s1, s2, s3, s4, s5, s6, s7, s8) != EOF) {

# Blank line
	if (s1 == "") {
	    printf ("\n", >>odbpath)
	    if (db) {
		printf ("\n")
		}
	    }

# Date and time if comment line
	else if (s1 == "#") {
	    if (first)
		printf ("# %s %s %s\n", gettime.dweek, gettime.date, gettime.time, >odbpath)
	    else
		printf ("# %s %s %s\n", gettime.dweek, gettime.date, gettime.time, >>odbpath)
	    first = no
	    if (db) {
		printf ("# %s %s %s\n", gettime.dweek, gettime.date, gettime.time)
		}
	    }

#	else if (ncoeff > 0 && s2 == "") {
#	    printf (\t\t%s\n", s1)
#	    }

# Two-token lines are parameter assignments
	else if (s3 == "") {
	    if (s1 == "id") {
		printf ("\tid\t%s\n", odbroot, >>odbpath)
		if (db) {
		    printf ("\tid\t%s\n", odbroot)
		    }
		}
	    else if (s1 == "image") {
		printf ("\timage\t%s\n", odbroot, >>odbpath)
		if (db) {
		    printf ("\timage\t%s\n", odbroot)
		    }
		}
	    else if (s1 == "slope") {
		printf ("\tslope\t%s\n", s2, >>odbpath)
		if (db) {
		    printf ("\tslope\t%s\n", s2)
		    }
		if (!keepfit) {
		    break
		    }
		}
	    else if (s1 == "coefficents") {
		icoeff=0
		ncoeff = int (s2)
		}
	    else {
		printf ("\t%s\t%s\n", s1, s2, >>odbpath)
		if (db) {
		    printf ("\t%s\t%s\n", s1, s2)
		    }
		}
	    }

# Three-token lines
	else if (s4 == "") {
	    printf ("begin\tecidentify %s\n", odbroot, >>odbpath)
	    if (db) {
		printf ("begin\tecidentify %s\n", odbroot)
		}
	    }

# Calibration emission lines: s3 is line center pixel coordinate
	else {
	    if (shift!= 0.0) {
		x = (real (s3) * pscale) + pshift
		}
	    else {
		x = real (s3)
		}
	    w1 = real (s4)
	    w2 = real (s5)
	    i = int (s1)
	    io = int (s2)
	    printf ("\t\t%2d %4d %10.4f %11.5f  %10.4f   %s  %s  %s\n", i, io, x, w1, w2, s6, s7, s8, >>odbpath)
	    if (db) {
		printf ("\t\t%2d %4d %10.4f %11.5f  %10.4f   %s  %s  %s\n", i, io, x, w1, w2, s6, s7, s8)
		}
	    }
	s1 = ""
	s2 = ""
	s3 = ""
	s4 = ""
	s5 = ""
	s6 = ""
	s7 = ""
	s8 = ""
	}
end

# Jan 22 2008	New program to copy a database echelle dispersion file
# Jan 25 2008	Add debugging code; break after slope line
# Jan 28 2008	Add keepfit parameter to simply copy database entry
# Feb  1 2008	Print processing date and time instead of original date and time
# Mar 20 2008	Null out all strings after each line is read
# May 16 2008	Add input time argument to gettime() 
# Jul 18 2008	Simply copy ID database file if no shift and same file name
# Jul 30 2008	Print amount of shift if nonzero
# Aug 14 2008	If |shift| is less than 0.000001, set it to 0.00000
# Aug 15 2008	If shift is non-zero, add shift database entry before slope
# Sep  9 2008	If first # line, send to new file, else append
# Sep  9 2008	Print 4 decimal places instead of 2 for shifted line centers
# Dec 15 2008	Add timediff argument to GETTIME to get current time
